package ctbrec.ui.sites.cherrytv;

import com.apollographql.apollo.ApolloClient;
import ctbrec.Config;
import ctbrec.Model;
import ctbrec.io.HtmlParser;
import ctbrec.io.HttpException;
import ctbrec.sites.cherrytv.CherryTv;
import ctbrec.sites.cherrytv.CherryTvModel;
import ctbrec.ui.tabs.PaginatedScheduledService;
import javafx.concurrent.Task;
import okhttp3.Request;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.jsoup.nodes.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.Objects;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import static ctbrec.Model.State.*;
import static ctbrec.io.HttpConstants.ACCEPT_LANGUAGE;
import static ctbrec.io.HttpConstants.USER_AGENT;

public class CherryTvUpdateService extends PaginatedScheduledService {

    private static final Logger LOG = LoggerFactory.getLogger(CherryTvUpdateService.class);
    private String url;
    private final CherryTv site;
    private ApolloClient apolloClient;

    public CherryTvUpdateService(String url, CherryTv site) {
        this.site = site;
        this.url = url;

        ExecutorService executor = Executors.newSingleThreadExecutor(r -> {
            var t = new Thread(r);
            t.setDaemon(true);
            t.setName("CherryTvUpdateService");
            return t;
        });
        setExecutor(executor);

        apolloClient = ApolloClient.builder()
                .serverUrl(site.getBaseUrl() + "/graphql")
                .build();
    }

    @Override
    protected Task<List<Model>> createTask() {
        return new Task<>() {
            @Override
            public List<Model> call() throws IOException {
                String pageUrl = CherryTvUpdateService.this.url;
                LOG.debug("Fetching page {}", pageUrl);
                apolloClient.

                var request = new Request.Builder()
                        .url(pageUrl)
                        .header(ACCEPT_LANGUAGE, Locale.ENGLISH.getLanguage())
                        .header(USER_AGENT, Config.getInstance().getSettings().httpUserAgent)
                        .build();
                try (var response = site.getHttpClient().execute(request)) {
                    if (response.isSuccessful()) {
                        return parseModels(Objects.requireNonNull(response.body()).string());
                    } else {
                        throw new HttpException(response.code(), response.message());
                    }
                }
            }
        };
    }

    private List<Model> parseModels(String body) {
        var json = new JSONObject(body);
        // LOG.debug(json.toString(2));
        List<Model> models = new ArrayList<>();
        try {
            JSONArray broadcasts = json.getJSONObject("data").getJSONObject("broadcasts").getJSONArray("broadcasts");
            for (int i = 0; i < broadcasts.length(); i++) {
                JSONObject broadcast = broadcasts.getJSONObject(i);
                CherryTvModel model = site.createModel(broadcast.optString("username"));
                model.setDisplayName(broadcast.optString("title"));
                model.setDescription(broadcast.optString("description"));
                model.setPreview(broadcast.optString("thumbnailUrl"));
                var online = broadcast.optString("showStatus").equalsIgnoreCase("Public")
                        && broadcast.optString("broadcastStatus").equalsIgnoreCase("Live");
                model.setOnline(online);
                model.setOnlineState(online ? ONLINE : OFFLINE);
                JSONArray tags = broadcast.optJSONArray("tags");
                if (tags != null) {
                    for (int j = 0; j < tags.length(); j++) {
                        model.getTags().add(tags.getString(j));
                    }
                }
                models.add(model);
            }
        } catch (JSONException e) {
            LOG.error("Couldn't parse JSON, the structure might have changed", e);
        }
        return models;
    }

    public void setUrl(String url) {
        this.url = url;
    }

}
